/*
 *	spButtonP.h
 */

#ifndef __SPBUTTONP_H
#define __SPBUTTONP_H

#include <sp/spLabelP.h>
#include <sp/spButton.h>
#if defined(GTK)
#include <sp/spButton_Gtk.h>
#elif defined(_WIN32) && !defined(USE_MOTIF)
#include <sp/spButton_Win.h>
#elif defined(ANDROID)
#include <sp/spButton_Android.h>
#elif defined(COCOA)
#include <sp/spButton_Cocoa.h>
#elif defined(MACOS)
#include <sp/spButton_Mac.h>
#else
#include <sp/spButton_Xm.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define SP_MAX_BUTTON_LABEL 512

typedef struct _spButtonPart {
    spBool set;				/* is toggle button set? */
    spBool default_flag;		/* is default button? */
    spBool cancel_flag;			/* is cancel button? */
    spBool radio_end;			/* is end of radio button? */
    spBool destructive_flag;		/* is destructive button? */
} spButtonPart;

typedef struct _spButtonRec {
    spObjectPart object;
    spComponentPart component;
    spPrimitivePart primitive;
    spLabelPart label;
    spButtonPart button;
} spButtonRec;

typedef struct _spButtonClassPart {
    int dummy;
} spButtonClassPart;

typedef struct _spButtonClassRec {
    spObjectClassPart object;
    spComponentClassPart component;
    spPrimitiveClassPart primitive;
    spLabelClassPart label;
    spButtonClassPart button;
} spButtonClassRec;

#if defined(MACOS)
#pragma import on
#endif

extern spButtonClassRec SpButtonClassRec;

#define SpButtonPart(comp) (((spButton)comp)->button)
#define SpParentButtonPart(comp) (((spButton)SpGetParent(comp))->button)
#define SpGetButtonClass(comp) ((spButtonClass)((comp)->object.object_class))

extern int spGetMnemonic(const char *label, char *text);
extern spBool spGetNonMnemonicTitle(spComponent component, char *label);
extern void spButtonPartInit(spObject object);
extern void spButtonPartFree(spObject object);
extern spBool spButtonCreate(spObject object);
extern void spButtonCreateArch(spComponent component);
extern spBool spButtonSetParams(spObject object);
extern void spButtonSetParamsArch(spComponent component);
extern spBool spButtonGetParams(spObject object);
extern void spSetToggleStateArch(spComponent component);
extern void spGetToggleStateArch(spComponent component);
  
extern spComponent spGetRadioGroupPrev(spComponent component);
extern spComponent spGetRadioGroupNext(spComponent component);
extern spComponent spGetRadioGroupFirst(spComponent component);
extern spComponent spGetRadioGroupLast(spComponent component);
extern void spAddRadioGroup(spComponent component, spBool radio_end);
extern void spCheckRadioButtonCB(spComponent component, void *data);

#if defined(MACOS)
#pragma import off
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPBUTTONP_H */
