/*
 *	spDrawP.h
 */

#ifndef __SPDRAWP_H
#define __SPDRAWP_H

#include <sp/spPrimitiveP.h>
#include <sp/spDIBitmap.h>
#include <sp/spDraw.h>
#if defined(GTK)
#include <sp/spDraw_Gtk.h>
#elif defined(_WIN32) && !defined(USE_MOTIF)
#include <sp/spDraw_Win.h>
#elif defined(ANDROID)
#include <sp/spDraw_Android.h>
#elif defined(COCOA)
#include <sp/spDraw_Cocoa.h>
#elif defined(MACOS)
#include <sp/spDraw_Mac.h>
#else
#include <sp/spDraw_Xm.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _spPluginPolygonInstance *spPluginPolygonInstance;
    
struct _spPolygon {
    spPolygonArchPart arch_part;
    int max_point;
    int num_point;
    spBool end_flag;
    int px;
    int py;
    int move_dx;
    int move_dy;

    spPluginPolygonInstance instance;	/* for plugin */
};

#define SpPolygonArch(obj) ((obj)->arch_part)
    
#if defined(MACOS)
#pragma import on
#endif

extern void *spGetImageHandleArch(spComponent component);
extern spBool spCopyImageArch(spComponent src, spComponent dest,
			      int src_x, int src_y, int width, int height,
			      int dest_x, int dest_y);
extern spBool spStretchImageArch(spComponent src, spComponent dest,
				 int src_x, int src_y, int src_width, int src_height,
				 int dest_x, int dest_y, int dest_width, int dest_height);
extern spBool spStretchImageDefault(spComponent src, spComponent dest,
				    int src_x, int src_y, int src_width, int src_height,
				    int dest_x, int dest_y, int dest_width, int dest_height);

extern void spFillRectangleArch(spComponent component, spGraphics graphics,
				int x, int y, int width, int height);
extern void spDrawRectangleArch(spComponent component, spGraphics graphics,
				int x, int y, int width, int height);
extern double spConvertArcAngleToSquareArc(double yfactor, double angle);
extern void spFillArcArch(spComponent component, spGraphics graphics,
			  int x, int y, int width, int height, double angle1, double angle2);
extern void spDrawArcArch(spComponent component, spGraphics graphics,
			  int x, int y, int width, int height, double angle1, double angle2);
extern void spDrawLineArch(spComponent component, spGraphics graphics,
			   int x1, int y1, int x2, int y2);
extern void spDrawPointArch(spComponent component, spGraphics graphics,
			    int x, int y);
extern void spDrawStringArch(spComponent component, spGraphics graphics,
			     int x, int y, const char *string);
extern spBool spGetStringExtentArch(spComponent component, spGraphics graphics, const char *string,
				    int *x, int *y, int *width, int *height, int *next_x);
    
extern spBool spCreatePolygonArch(spPolygon polygon);
extern spBool spDestroyPolygonArch(spPolygon polygon);
extern spBool spStartPolygonPointArch(spPolygon polygon, int x, int y);
extern spBool spAddPolygonPointArch(spPolygon polygon, int x, int y);
extern spBool spAddPolygonRelativePointArch(spPolygon polygon, int dx, int dy);
extern spBool spEndPolygonPointArch(spPolygon polygon);
extern spBool spMovePolygonArch(spPolygon polygon, int dx, int dy);
extern spBool spFillPolygonArch(spComponent component, spGraphics graphics, spPolygon polygon);
extern spBool spDrawPolygonArch(spComponent component, spGraphics graphics, spPolygon polygon);

extern spBool spGetPolygonPointArch(spPolygon polygon, int index, int *x, int *y);
extern spBool spGetPolygonPoint(spPolygon polygon, int index, int *x, int *y, spBool before_move);
    
extern unsigned long spGetDrawingCapsArch(spComponent component);
    
extern spBool spForwardPage(spComponent component);
extern spBool spSetCurrentPage(spComponent component, int current_page); /* 1-based page number (0: reset). must be called before spOpenDrawingTargetDialog */

#if defined(MACOS)
#pragma import off
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPDRAWP_H */
