/*
 *	spGraphics_Gtk.h
 */

#ifndef __SPGRAPHICS_GTK_H
#define __SPGRAPHICS_GTK_H

#include <gtk/gtk.h>
#include <gdk/gdk.h>

#if !GTK_CHECK_VERSION(1,2,0)
#include <gdk/gdkx.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define SP_SYSTEM_COLOR_BACKGROUND_PIXEL spRGB(215,215,215)
#define SP_SYSTEM_COLOR_TOP_SHADOW_PIXEL spRGB(255,255,255)
#define SP_SYSTEM_COLOR_BOTTOM_SHADOW_PIXEL spRGB(150,150,150)
#define SP_SYSTEM_COLOR_HIGHLIGHT_PIXEL spRGB(255,255,255)
#define SP_SYSTEM_COLOR_DARK_SHADOW_PIXEL 0L
#define SP_SYSTEM_COLOR_BLACK_PIXEL 0L
#define SP_SYSTEM_COLOR_WHITE_PIXEL spRGB(255,255,255)
#define SP_SYSTEM_COLOR_FOCUS_PIXEL 0L
#define SP_SYSTEM_COLOR_TAB_BACKGROUND_PIXEL SP_SYSTEM_COLOR_BACKGROUND_PIXEL
#define SP_SYSTEM_COLOR_PROGRESS_BAR_PIXEL spRGB(10,36,106)
#define SP_SYSTEM_COLOR_LABEL_STRING_PIXEL 0L

typedef struct _spCursorArchPart {
    GdkCursor *cursor;
} spCursorArchPart;

typedef struct _spGraphicsArchPart {
#if GTK_CHECK_VERSION(2,0,0)
    PangoFontDescription *font_desc;
#else
    GdkFont *font;
#endif

#if GTK_CHECK_VERSION(3,0,0)
#if GTK_CHECK_VERSION(3,4,0)
    GdkRGBA rgba;
#else
    GdkColor color;
#endif
    cairo_operator_t op;
#else
    GdkGC *gc;
#endif
} spGraphicsArchPart;

#if GTK_CHECK_VERSION(3,0,0)
extern spBool spUpdateLineContextCairo(cairo_t *cr, spGraphics graphics);
extern spBool spUpdateColorCairo(cairo_t *cr, spGraphics graphics);
#endif

#if GTK_CHECK_VERSION(3,4,0)
extern spBool spGetColorValueGtk(const char *color_name, GdkRGBA *color);
extern spBool spPixelToGdkRGBA(spPixel pixel, GdkRGBA *color);
#else
extern spBool spGetColorValueGtk(const char *color_name, GdkColor *color);
extern spBool spPixelToGdkColor(spPixel pixel, GdkColor *color);
#endif
    
#if GTK_CHECK_VERSION(2,0,0)
extern spBool spFontNameToFontDescriptionGtk(const char *font_name, PangoFontDescription *font_desc);
extern spBool spFontDescriptionToFontNameGtk(PangoFontDescription *font_desc, char *font_name, int buf_size);
#endif

#if !GTK_CHECK_VERSION(1,2,0)
extern const char **spGetFontFamilyListX11(Display *display);
extern spBool spIsFontSizeSupportedX11(Display *display, int family, unsigned long style, int size);
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPGRAPHICS_GTK_H */
