/*
 *	spPluginCanvasP.h
 */

#ifndef __SPPLUGINCANVASP_H
#define __SPPLUGINCANVASP_H

#include <sp/spPluginCanvas.h>
#include <sp/spPluginP.h>
#include <sp/spComponentP.h>
#include <sp/spDrawingPluginP.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _spPluginCanvasPart {
    /* private */
    spPlugin *plugin;
    int current_page;           /* 1-based */
    int num_page;
    spBool setup_dialog_called;
    spBool drawing_started;

    /* parameters */
    char *plugin_name;
    spCallbackFunc draw_func;		/* function pointer for drawable */
    void *draw_data;			/* data for drawable's callback */
    spPaperSize paper_size;
    spPaperOrientation paper_orientation;
    double left_margin_mm;
    double right_margin_mm;
    double top_margin_mm;
    double bottom_margin_mm;
} spPluginCanvasPart;

typedef struct _spPluginCanvasRec {
    spObjectPart object;
    spComponentPart component;
    spPluginCanvasPart plugin_canvas;
} spPluginCanvasRec;

typedef struct _spPluginCanvasClassPart {
    int dummy;
} spPluginCanvasClassPart;

typedef struct _spPluginCanvasClassRec {
    spObjectClassPart object;
    spComponentClassPart component;
    spPluginCanvasClassPart plugin_canvas;
} spPluginCanvasClassRec;

#if defined(MACOS)
#pragma import on
#endif

extern spPluginCanvasClassRec SpPluginCanvasClassRec;

#define SpPluginCanvasPart(comp) (((spPluginCanvas)comp)->plugin_canvas)
#define SpPluginCanvasDrawingRec(comp) ((spDrawingPluginRec *)((spPluginCanvas)comp)->plugin_canvas.plugin->rec)
#define SpPluginCanvasInstance(comp) (((spPluginCanvas)comp)->plugin_canvas.plugin->instance)

extern void spPluginCanvasPartInit(spObject object);
extern void spPluginCanvasPartFree(spObject object);
extern spBool spPluginCanvasCreate(spObject object);
extern spBool spPluginCanvasDestroy(spObject object);
extern spBool spPluginCanvasSetSize(spComponent component, int width, int height);
extern spBool spPluginCanvasGetSize(spComponent component, int *width, int *height);
extern spBool spPluginCanvasGetClientSize(spComponent component, int *width, int *height);
extern spBool spPluginCanvasSetSensitive(spComponent component, spBool flag);

extern void *spGetPluginPolygonInstance(spPolygon polygon, spPlugin *plugin, spBool *new_flag);
extern spBool spDestroyPluginPolygonInstanceInPlugin(spPlugin *plugin);
extern spBool spDestroyPluginPolygonInstance(spPolygon polygon);
extern void *spGetPluginCanvasPolygonInstance(spComponent component, spPolygon polygon);
    
#if defined(MACOS)
#pragma import off
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPPLUGINCANVASP_H */
