/*
 *	spTopLevel_Gtk.h
 */

#ifndef __SPTOPLEVEL_GTK_H
#define __SPTOPLEVEL_GTK_H

#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include <sp/spComponent_Gtk.h>

#ifdef __cplusplus
extern "C" {
#endif

#if GTK_CHECK_VERSION(2,0,0) && GLIB_CHECK_VERSION(2,2,6)
#define SP_GTK_DBUS_FIND_APPLICATION
#define SP_GTK_DBUS_RUN_METHOD_NAME "Run"
#define SP_GTK_DBUS_ADDRESS_FORMAT "unix:abstract=%s"
#endif

#if !GTK_CHECK_VERSION(2,0,0)
#define SP_GTK_USE_MAIN_LOOP_MUTEX
#define SP_GTK_USE_DRAW_MUTEX
#else
#undef SP_GTK_USE_MAIN_LOOP_MUTEX
#undef SP_GTK_USE_DRAW_MUTEX
#endif
    
#if GTK_CHECK_VERSION(2,0,0)
#define SP_GTK_USE_ASYNC_EVENTS_QUEUE
#else
#undef SP_GTK_USE_ASYNC_EVENTS_QUEUE
#endif

#if !defined(SP_GTK_USE_ASYNC_EVENTS_QUEUE)
#define SP_USE_DEFAULT_USER_EVENT    
#endif
    
typedef struct _spTopLevelArchPart {
#if GTK_CHECK_VERSION(3,0,0)
    GdkVisual *visual;
#else
    GdkWindow *root_window;
    gint root_depth;
    GdkPixmap *pixmap;
    GdkColormap *colormap;
#endif
    GdkColor black;
    GdkColor white;
    GdkColor gray;
#if GTK_CHECK_VERSION(2,0,0)
    PangoFontDescription *font_desc;
#else
    GdkFont *font;
#endif
    
#if GTK_CHECK_VERSION(3,0,0)
    cairo_t *cr;
#else
    GdkGC *fg_gc;
    GdkGC *bg_gc;
#endif

#if GTK_CHECK_VERSION(2,0,0)
    PangoContext *context;
#endif
    
    spBool unity_used;
    spBool menu_proxy_used;
    
#if defined(SP_GTK_USE_ASYNC_EVENTS_QUEUE)
    GAsyncQueue *events_queue;
    GSource *events_source;
#else
    gint main_idle_func_id;
    spUserEventDefault user_event;
#endif

#if defined(SP_GTK_DBUS_FIND_APPLICATION)
    gchar *dbus_interface_name;
    gchar *dbus_object_name;
    gchar *dbus_address;
    GDBusNodeInfo *dbus_introspection_data;
    GDBusServer *dbus_server;
#endif
} spTopLevelArchPart;

extern void spMainQuitGtk(void);
extern gint spMainLoopGtk(GtkWidget *widget, spBool dialog_flag);
extern spBool spLockDrawMutexGtk(void);
extern spBool spUnlockDrawMutexGtk(void);

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPTOPLEVEL_GTK_H */
